/*
 * 3D Studio to OpenGL File Converter 
 * by David Farrell, fdavid@cyberramp.net
 */

#ifdef WIN32
#include <windows.h>
#endif

#include "GL/gl.h"
#include "camera.h"

/* Module Globals */
GLfloat m[16];
GLfloat localm[16];
int movespeed = 10;
int rollspeed = 3;

/* Internal Function Calls */
void world_to_local() { glGetFloatv(GL_MODELVIEW_MATRIX, m); glLoadIdentity(); }
void local_to_world() { glMultMatrixf(m); }
void local()          { glPushMatrix(); glLoadMatrixf(localm); }
void world()          { glGetFloatv(GL_MODELVIEW_MATRIX, localm); glPopMatrix(); }

/* Pre-concatenates matrices: M*M^1*T*M = I*T*M = T*M, the camera's new position. */
void moveLeft()    { world_to_local(); glTranslatef( movespeed, 0, 0); local_to_world(); }
void moveRight()   { world_to_local(); glTranslatef(-movespeed, 0, 0); local_to_world(); }
void moveUp()      { world_to_local(); glTranslatef( 0,-movespeed, 0); local_to_world(); }
void moveDown()    { world_to_local(); glTranslatef( 0, movespeed, 0); local_to_world(); }
void moveForward() { world_to_local(); glTranslatef( 0, 0, movespeed); local_to_world(); }
void moveBack()    { world_to_local(); glTranslatef( 0, 0,-movespeed); local_to_world(); }

/* Pre-concatenates matrices: M*M^1*R*M = I*R*M = R*M, the camera's new orientation. */
void pitchUp()     { world_to_local(); glRotatef(-rollspeed, 1, 0, 0); local_to_world(); }
void pitchDown()   { world_to_local(); glRotatef( rollspeed, 1, 0, 0); local_to_world(); }
void yawRight()    { world_to_local(); glRotatef(-rollspeed, 0, 1, 0); local_to_world(); }
void yawLeft()     { world_to_local(); glRotatef( rollspeed, 0, 1, 0); local_to_world(); }
void rollLeft()    { world_to_local(); glRotatef(-rollspeed, 0, 0, 1); local_to_world(); }
void rollRight()   { world_to_local(); glRotatef( rollspeed, 0, 0, 1); local_to_world(); }

/* Modifies the local model matrix */
void ModelPitchUp()     { local(); glRotatef(-rollspeed, 1, 0, 0); world(); }
void ModelPitchDown()   { local(); glRotatef( rollspeed, 1, 0, 0); world(); }
void ModelYawRight()    { local(); glRotatef(-rollspeed, 0, 1, 0); world(); }
void ModelYawLeft()     { local(); glRotatef( rollspeed, 0, 1, 0); world(); }
void ModelRollLeft()    { local(); glRotatef(-rollspeed, 0, 0, 1); world(); }
void ModelRollRight()   { local(); glRotatef( rollspeed, 0, 0, 1); world(); }

void SetMovementSpeed(int s) { movespeed = s; }
void SetRollSpeed(int r)     { rollspeed = r; }
void ModelCoordinates()      { glPushMatrix(); glMultMatrixf(localm); }
void WorldCoordinates()      { glPopMatrix(); }
 
void SetCameraPosition(int x, int y, int z) { 
  glLoadIdentity(); 
  glTranslatef(x, y, z); 
}

void CameraReset() { 
  glLoadIdentity(); 
  glGetFloatv(GL_MODELVIEW_MATRIX, localm); 
  glTranslatef(0, 0, -200); 
}
















